@@ -33,7 +33,6 @@
 #include <glib/gi18n.h>
 #include <adwaita.h>
 
-#include "gtk3-to-4.h"
 #include "chatty-window.h"
 #include "chatty-manager.h"
 #include "chatty-purple.h"
@@ -168,7 +167,7 @@ chatty_application_show_about (GSimpleAction *action,
                          "comments", _("An SMS and XMPP messaging client"),
                          "website", "https://gitlab.gnome.org/World/Chatty",
                          "copyright", "© 2018–2023 Purism SPC",
-                         "issue-url", "https://gitlab.gnome.org/example/example/-/issues/new",
+                         "issue-url", "https://gitlab.gnome.org/World/Chatty/-/issues/new",
                          "license-type", GTK_LICENSE_GPL_3_0,
                          "developers", authors,
                          "designers", artists,
@@ -187,22 +186,25 @@ show_help_cb (GObject      *object,
 
   g_assert (CHATTY_IS_APPLICATION (self));
 
-  gtk_show_uri_full_finish (GTK_WINDOW (self->main_window), result, &error);
+  gtk_uri_launcher_launch_finish (GTK_URI_LAUNCHER (object), result, &error);
+
+  g_clear_object (&object);
 
   if (error) {
-      GtkWidget *message_dialog;
+    GtkWidget *dialog;
+    GtkWindow *window;
 
-      message_dialog = gtk_message_dialog_new (GTK_WINDOW (self->main_window),
-                                               GTK_DIALOG_DESTROY_WITH_PARENT,
-                                               GTK_MESSAGE_ERROR,
-                                               GTK_BUTTONS_CLOSE,
-                                               _("There was an error displaying help:\n%s"),
-                                               error->message);
-      g_signal_connect (message_dialog, "response",
-                        G_CALLBACK (gtk_window_destroy),
-                        NULL);
+    window = gtk_application_get_active_window (GTK_APPLICATION (g_application_get_default ()));
+    dialog = adw_message_dialog_new (window, _("Error"), NULL);
+    adw_message_dialog_format_body (ADW_MESSAGE_DIALOG (dialog),
+                                    _("There was an error displaying help:\n%s"),
+                                     error->message);
+    adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "close", _("Close"));
 
-      gtk_widget_set_visible (message_dialog, TRUE);
+    adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "close");
+    adw_message_dialog_set_close_response (ADW_MESSAGE_DIALOG (dialog), "close");
+
+    gtk_window_present (GTK_WINDOW (dialog));
   }
 }
 
@@ -212,11 +214,14 @@ chatty_application_show_help (GSimpleAction *action,
 			      gpointer       user_data)
 {
   ChattyApplication *self = CHATTY_APPLICATION (user_data);
+  GtkUriLauncher *uri_launcher = NULL;
+  GtkWindow *window;
+
   const char *url = "help:chatty";
 
-  gtk_show_uri_full (GTK_WINDOW (self->main_window), url,
-		     GDK_CURRENT_TIME, NULL,
-                     show_help_cb, self);
+  window = gtk_application_get_active_window (GTK_APPLICATION (g_application_get_default ()));
+  uri_launcher = gtk_uri_launcher_new (url);
+  gtk_uri_launcher_launch (uri_launcher, window, NULL, show_help_cb, self);
 }
 
 
@@ -295,8 +300,10 @@ main_window_focus_changed_cb (ChattyApplication *self)
   if (has_focus)
     chat = chatty_application_get_active_chat (self);
 
-  if (chat)
+  if (chat) {
     chatty_chat_set_unread_count (chat, 0);
+    chatty_chat_withdraw_notification (chat);
+  }
 }
 
 static void
