@@ -14,7 +14,6 @@
 # include "config.h"
 #endif
 
-#include "gtk3-to-4.h"
 #include "chatty-utils.h"
 #include "chatty-attachment.h"
 #include "chatty-log.h"
@@ -24,7 +23,9 @@ struct _ChattyAttachment
   GtkBox       parent_instance;
 
   GtkWidget   *overlay;
+  GtkWidget   *label;
   GtkWidget   *remove_button;
+  GtkWidget   *spinner;
 
   GFile       *file;
 };
@@ -72,6 +73,7 @@ attachment_update_image (ChattyAttachment *self,
   g_assert (file);
 
   file_info = g_file_query_info (file,
+                                 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
                                  G_FILE_ATTRIBUTE_STANDARD_ICON ","
                                  G_FILE_ATTRIBUTE_THUMBNAIL_PATH,
                                  G_FILE_QUERY_INFO_NONE,
@@ -80,6 +82,7 @@ attachment_update_image (ChattyAttachment *self,
     g_warning ("Error querying info: %s", error->message);
 
   thumbnail = g_file_info_get_attribute_byte_string (file_info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH);
+  gtk_widget_set_visible (self->spinner, FALSE);
 
   if (thumbnail) {
     GtkWidget *frame;
@@ -92,12 +95,23 @@ attachment_update_image (ChattyAttachment *self,
     gtk_frame_set_child (GTK_FRAME (frame), image);
     gtk_overlay_set_child (GTK_OVERLAY (self->overlay), frame);
   } else {
-    GIcon *icon;
+    g_autofree char *file_mime_type = NULL;
 
     gtk_widget_set_margin_end (image, 6);
-    icon = (GIcon *)g_file_info_get_attribute_object (file_info, G_FILE_ATTRIBUTE_STANDARD_ICON);
-    gtk_image_set_from_gicon (GTK_IMAGE (image), icon);
-    image = gtk_image_new_from_gicon (icon);
+    file_mime_type = g_content_type_get_mime_type (g_file_info_get_content_type (file_info));
+
+    if (!file_mime_type)
+      gtk_image_set_from_icon_name (GTK_IMAGE (image), "text-x-generic-symbolic");
+    else if (strstr (file_mime_type, "vcard"))
+      gtk_image_set_from_icon_name (GTK_IMAGE (image), "contact-new-symbolic");
+    else if (strstr (file_mime_type, "calendar"))
+      gtk_image_set_from_icon_name (GTK_IMAGE (image), "x-office-calendar-symbolic");
+    else {
+      g_autoptr(GIcon) icon = NULL;
+
+      icon = g_content_type_get_symbolic_icon (file_mime_type);
+      gtk_image_set_from_gicon (GTK_IMAGE (image), icon);
+    }
     gtk_image_set_pixel_size (GTK_IMAGE (image), 96);
     gtk_overlay_set_child (GTK_OVERLAY (self->overlay), image);
   }
@@ -156,6 +170,8 @@ chatty_attachment_class_init (ChattyAttachmentClass *klass)
                                                "ui/chatty-attachment.ui");
 
   gtk_widget_class_bind_template_child (widget_class, ChattyAttachment, overlay);
+  gtk_widget_class_bind_template_child (widget_class, ChattyAttachment, label);
+  gtk_widget_class_bind_template_child (widget_class, ChattyAttachment, spinner);
   gtk_widget_class_bind_template_child (widget_class, ChattyAttachment, remove_button);
 
   gtk_widget_class_bind_template_callback (widget_class, attachment_remove_clicked_cb);
@@ -202,6 +218,7 @@ chatty_attachment_set_file (ChattyAttachment *self,
   g_set_object (&self->file, file);
 
   file_info = g_file_query_info (file,
+                                 G_FILE_ATTRIBUTE_STANDARD_NAME ","
                                  G_FILE_ATTRIBUTE_STANDARD_ICON ","
                                  G_FILE_ATTRIBUTE_THUMBNAIL_PATH ","
                                  G_FILE_ATTRIBUTE_THUMBNAIL_IS_VALID ","
@@ -221,11 +238,14 @@ chatty_attachment_set_file (ChattyAttachment *self,
   CHATTY_TRACE_MSG ("has thumbnail: %d, failed thumbnail: %d, valid thumbnail: %d",
                     !!thumbnail, thumbnail_failed, thumbnail_valid);
 
+  gtk_label_set_text (GTK_LABEL (self->label), g_file_info_get_name (file_info));
+
   if (thumbnail || (thumbnail_failed && thumbnail_valid)) {
     attachment_update_image (self, image, file);
   } else {
     AttachmentData *data;
 
+    gtk_widget_set_visible (self->spinner, TRUE);
     data = g_new0 (AttachmentData, 1);
     data->self = g_object_ref (self);
     data->image = g_object_ref (image);
