@@ -15,16 +15,18 @@
 
 #include <glib/gi18n.h>
 
-#include "gtk3-to-4.h"
 #include "chatty-purple.h"
 #include "chatty-contact.h"
 #include "chatty-contact-list.h"
 #include "chatty-chat.h"
+#include "chatty-enums.h"
 #include "chatty-ma-key-chat.h"
+#include "chatty-message.h"
 #include "chatty-avatar.h"
 #include "chatty-clock.h"
 #include "chatty-list-row.h"
 #include "chatty-contact-provider.h"
+#include "chatty-settings.h"
 
 struct _ChattyListRow
 {
@@ -199,10 +201,14 @@ chatty_list_row_update (ChattyListRow *self)
     chatty_item_get_avatar (self->item);
 
    if (*number) {
-      GtkGesture    *gesture;
-      gesture = gtk_gesture_long_press_new ();
-      gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (gesture));
-      g_signal_connect (gesture, "pressed", G_CALLBACK (long_pressed), self);
+     GtkGesture *gesture;
+     gesture = gtk_gesture_long_press_new ();
+     /*
+      * gtk_widget_add_controller () transfers ownership of the gesture to
+      * ChattyListRow so you will not have to worry about freeing it manually
+      */
+     gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (gesture));
+     g_signal_connect (gesture, "pressed", G_CALLBACK (long_pressed), self);
    }
 
   } else if (CHATTY_IS_CHAT (self->item) && !self->hide_chat_details) {
@@ -217,6 +223,7 @@ chatty_list_row_update (ChattyListRow *self)
 
     gtk_widget_set_visible (self->subtitle, last_message && *last_message);
     if (last_message && *last_message) {
+      ChattySettings *settings;
       g_autofree char *message_stripped = NULL;
 
 #ifdef PURPLE_ENABLED
@@ -226,7 +233,72 @@ chatty_list_row_update (ChattyListRow *self)
 #endif
       g_strstrip (message_stripped);
 
-      gtk_label_set_label (GTK_LABEL (self->subtitle), message_stripped);
+      settings = chatty_settings_get_default ();
+
+      if (chatty_settings_get_strip_url_tracking_ids (settings)) {
+        g_autofree char *utm_stripped_message = NULL;
+
+        utm_stripped_message = chatty_utils_strip_utm_from_message (message_stripped);
+        gtk_label_set_label (GTK_LABEL (self->subtitle), utm_stripped_message);
+      } else
+        gtk_label_set_label (GTK_LABEL (self->subtitle), message_stripped);
+
+    } else {
+      GListModel *model;
+      guint n_items;
+
+      model = chatty_chat_get_messages (item);
+      n_items = g_list_model_get_n_items (model);
+      if (n_items > 0) {
+        g_autoptr(ChattyMessage) message = NULL;
+        ChattyMsgType message_type = CHATTY_MESSAGE_UNKNOWN;
+        const char *message_text = "";
+
+        message = g_list_model_get_item (model, n_items - 1);
+        message_type = chatty_message_get_msg_type (message);
+        switch (message_type) {
+          case CHATTY_MESSAGE_IMAGE:
+            message_text = _("Picture");
+          break;
+          case CHATTY_MESSAGE_VIDEO:
+            message_text = _("Video");
+          break;
+          case CHATTY_MESSAGE_AUDIO:
+            message_text = _("Audio");
+          break;
+          case CHATTY_MESSAGE_LOCATION:
+            message_text = _("Location");
+          break;
+          case CHATTY_MESSAGE_FILE:
+            message_text = _("File");
+          break;
+          case CHATTY_MESSAGE_MMS:
+            message_text = _("MMS Message");
+          break;
+          case CHATTY_MESSAGE_UNKNOWN:
+          case CHATTY_MESSAGE_TEXT:
+          case CHATTY_MESSAGE_HTML:
+          case CHATTY_MESSAGE_HTML_ESCAPED:
+          case CHATTY_MESSAGE_MATRIX_HTML:
+          default:
+          break;
+        }
+        if (!*message_text) {
+          GList *files = NULL;
+          files = chatty_message_get_files (message);
+          if (files && g_list_length (files) == 1)
+            message_text = _("File");
+          else if (files)
+            message_text = _("Multiple Files");
+        }
+        if (*message_text) {
+          g_autofree char *markup = NULL;
+
+          markup = g_strdup_printf ("<span style=\"italic\">\%s</span>", message_text);
+          gtk_widget_set_visible (self->subtitle, TRUE);
+          gtk_label_set_markup (GTK_LABEL (self->subtitle), markup);
+        }
+      }
     }
 
     unread_count = chatty_chat_get_unread_count (item);
@@ -311,13 +383,17 @@ static void
 chatty_list_row_call_button_clicked_cb (ChattyListRow *self)
 {
   g_autofree char *uri = NULL;
+  g_autoptr(GtkUriLauncher) uri_launcher = NULL;
+  GtkWindow *window;
 
   g_return_if_fail (CHATTY_IS_CONTACT (self->item));
 
   uri = g_strconcat ("tel://", chatty_item_get_username (self->item), NULL);
 
   g_debug ("Calling uri: %s", uri);
-  gtk_show_uri (NULL, uri, GDK_CURRENT_TIME);
+  window = gtk_application_get_active_window (GTK_APPLICATION (g_application_get_default ()));
+  uri_launcher = gtk_uri_launcher_new (uri);
+  gtk_uri_launcher_launch (uri_launcher, window, NULL, NULL, NULL);
 }
 
 static void
