@@ -45,6 +45,7 @@ struct _ChattyMessageBar
   guint          save_timeout_id;
   gboolean       draft_is_loading;
   gboolean       is_self_change;
+  char          *preedit;
 };
 
 
@@ -84,51 +85,47 @@ chatty_message_chat_encrypt_changed_cb (ChattyMessageBar *self)
 }
 
 static void
-message_bar_file_chooser_response_cb (ChattyMessageBar *self,
-                                      int               response_id,
-                                      GtkDialog        *dialog)
+message_bar_file_chooser_response_cb (GObject         *dialog,
+                                      GAsyncResult    *response,
+                                      gpointer        user_data)
+
 {
+  ChattyMessageBar *self = CHATTY_MESSAGE_BAR (user_data);
+  g_autoptr(GError) error = NULL;
+  g_autoptr(GFile) open_filename = NULL;
+
   g_assert (CHATTY_IS_MESSAGE_BAR (self));
+  open_filename = gtk_file_dialog_open_finish (GTK_FILE_DIALOG (dialog), response, &error);
+  g_clear_object (&dialog);
 
-  if (response_id == GTK_RESPONSE_ACCEPT) {
-    g_autoptr(GFile) file = NULL;
-
-    file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
-
-    chatty_attachments_bar_add_file (CHATTY_ATTACHMENTS_BAR (self->attachment_bar), file);
-
-    /* Currently multiple files are allowed only for MMS chats */
-    gtk_widget_set_sensitive (self->send_file_button, CHATTY_IS_MM_CHAT (self->chat));
-    /* Files with message content is supported only by MMS chats */
-    gtk_widget_set_sensitive (self->message_input, CHATTY_IS_MM_CHAT (self->chat));
+  if (!open_filename) {
+    if (!g_error_matches (error, GTK_DIALOG_ERROR, GTK_DIALOG_ERROR_DISMISSED))
+      g_warning ("Error getting file: %s", error->message);
+    return;
   }
 
-  gtk_window_destroy (GTK_WINDOW (dialog));
+  chatty_attachments_bar_add_file (CHATTY_ATTACHMENTS_BAR (self->attachment_bar), open_filename);
+
+  /* Currently multiple files are allowed only for MMS chats */
+  gtk_widget_set_sensitive (self->send_file_button, CHATTY_IS_MM_CHAT (self->chat));
+  /* Files with message content is supported only by MMS chats */
+  gtk_widget_set_sensitive (self->message_input, CHATTY_IS_MM_CHAT (self->chat));
 }
 
 static void
 chat_page_show_file_chooser (ChattyMessageBar *self)
 {
   GtkWindow *window;
-  GtkWidget *dialog;
+  GtkFileDialog *dialog;
 
   g_assert (CHATTY_IS_MESSAGE_BAR (self));
 
   g_debug ("Show file chooser");
 
   window = gtk_application_get_active_window (GTK_APPLICATION (g_application_get_default ()));
-  dialog = gtk_file_chooser_dialog_new (_("Select File..."),
-                                        window,
-                                        GTK_FILE_CHOOSER_ACTION_OPEN,
-                                        _("Cancel"), GTK_RESPONSE_CANCEL,
-                                        _("Open"), GTK_RESPONSE_ACCEPT,
-                                        NULL);
+  dialog = gtk_file_dialog_new ();
 
-  g_signal_connect_object (dialog, "response",
-                           G_CALLBACK (message_bar_file_chooser_response_cb),
-                           self, G_CONNECT_SWAPPED);
-
-  gtk_window_present (GTK_WINDOW (dialog));
+  gtk_file_dialog_open (dialog, window, NULL, message_bar_file_chooser_response_cb, self);
 }
 
 static void
@@ -206,7 +203,9 @@ chat_page_save_message_to_db (gpointer user_data)
   ChattyMessageBar *self = user_data;
   g_autoptr(ChattyMessage) draft = NULL;
   g_autofree char *text = NULL;
+  g_autofree char *message_buffer_text = NULL;
   g_autofree char *uid = NULL;
+  int cursor_location = 0;
 
   g_assert (CHATTY_IS_MESSAGE_BAR (self));
 
@@ -215,7 +214,19 @@ chat_page_save_message_to_db (gpointer user_data)
 
   CHATTY_TRACE (chatty_chat_get_chat_name (self->chat), "Saving draft to");
 
-  g_object_get (self->message_buffer, "text", &text, NULL);
+  g_object_get (self->message_buffer,
+                "text",
+                &message_buffer_text,
+                "cursor-position",
+                &cursor_location,
+                NULL);
+  if (self->preedit && *self->preedit) {
+    GString *message_string = NULL;
+    message_string = g_string_new (message_buffer_text);
+    message_string = g_string_insert (message_string, cursor_location, self->preedit);
+    text = g_string_free (message_string, false);
+  } else
+    text = g_strdup (message_buffer_text);
 
   uid = g_uuid_string_random ();
   draft = chatty_message_new (NULL, text, uid, time (NULL),
@@ -320,6 +331,7 @@ view_send_message_async_cb (GObject      *object,
   g_autoptr(ChattyMessageBar) self = user_data;
 
   chatty_chat_set_unread_count (self->chat, 0);
+  chatty_chat_withdraw_notification (self->chat);
 }
 
 static void
@@ -328,15 +340,28 @@ message_bar_send_message_button_clicked_cb (ChattyMessageBar *self)
   ChattyAccount *account;
   g_autoptr(ChattyMessage) msg = NULL;
   g_autofree char *message = NULL;
+  g_autofree char *message_buffer_text = NULL;
   GtkTextIter    start, end;
   GList *files;
+  int cursor_location = 0;
 
   g_assert (CHATTY_IS_MESSAGE_BAR (self));
 
   files = chatty_attachments_bar_get_files (CHATTY_ATTACHMENTS_BAR (self->attachment_bar));
 
   gtk_text_buffer_get_bounds (self->message_buffer, &start, &end);
-  message = gtk_text_buffer_get_text (self->message_buffer, &start, &end, FALSE);
+  message_buffer_text = gtk_text_buffer_get_text (self->message_buffer, &start, &end, FALSE);
+  g_object_get (self->message_buffer,
+                "cursor-position",
+                &cursor_location,
+                NULL);
+  if (self->preedit && *self->preedit) {
+    GString *message_string = NULL;
+    message_string = g_string_new (message_buffer_text);
+    message_string = g_string_insert (message_string, cursor_location, self->preedit);
+    message = g_string_free (message_string, false);
+  } else
+    message = g_strdup (message_buffer_text);
 
 #ifdef PURPLE_ENABLED
   if (CHATTY_IS_PP_CHAT (self->chat) &&
@@ -345,6 +370,11 @@ message_bar_send_message_button_clicked_cb (ChattyMessageBar *self)
 
     gtk_widget_set_visible (self->send_message_button, FALSE);
     gtk_text_buffer_delete (self->message_buffer, &start, &end);
+    /* Make sure the text view things is necessary to reset the im context */
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (self->message_input), FALSE);
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (self->message_input), TRUE);
+    /* This clears preedit */
+    gtk_text_view_reset_im_context (GTK_TEXT_VIEW (self->message_input));
 
     return;
   }
@@ -356,7 +386,7 @@ message_bar_send_message_button_clicked_cb (ChattyMessageBar *self)
 
   gtk_widget_grab_focus (self->message_input);
 
-  if (gtk_text_buffer_get_char_count (self->message_buffer) || files) {
+  if (gtk_text_buffer_get_char_count (self->message_buffer) || (self->preedit && *self->preedit) || files) {
     g_autofree char *escaped = NULL;
 
 #ifdef PURPLE_ENABLED
@@ -383,8 +413,14 @@ message_bar_send_message_button_clicked_cb (ChattyMessageBar *self)
    * deleting the existing text, simply make the entry sensitive again. */
   if (CHATTY_IS_MA_CHAT (self->chat) && files)
     gtk_widget_set_sensitive (self->message_input, TRUE);
-  else
+  else {
     gtk_text_buffer_delete (self->message_buffer, &start, &end);
+    /* Make sure the text view things is necessary to reset the im context */
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (self->message_input), FALSE);
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (self->message_input), TRUE);
+    /* This clears preedit */
+    gtk_text_view_reset_im_context (GTK_TEXT_VIEW (self->message_input));
+  }
 
   {
     g_autoptr(ChattyMessage) draft = NULL;
@@ -401,6 +437,70 @@ message_bar_send_message_button_clicked_cb (ChattyMessageBar *self)
   g_clear_handle_id (&self->save_timeout_id, g_source_remove);
 }
 
+static void
+message_bar_text_buffer_clipboard_paste_done_cb (ChattyMessageBar *self)
+{
+  ChattySettings  *settings;
+  g_autofree char *stripped_message = NULL;
+  g_autofree char *message_buffer_text = NULL;
+  GtkTextIter      start, end;
+
+  settings = chatty_settings_get_default ();
+  if (!chatty_settings_get_strip_url_tracking_ids (settings)) {
+    chatty_settings_set_strip_url_tracking_ids_dialog (settings, FALSE);
+    return;
+  }
+
+  gtk_text_buffer_get_bounds (self->message_buffer, &start, &end);
+  message_buffer_text = gtk_text_buffer_get_text (self->message_buffer, &start, &end, FALSE);
+
+  stripped_message = chatty_utils_strip_utm_from_message (message_buffer_text);
+
+  if (g_strcmp0 (stripped_message, message_buffer_text) != 0) {
+
+    /* To not annoy the user, we only show the strip ID dialog the first time */
+    if (!chatty_settings_get_strip_url_tracking_ids_dialog (settings)) {
+      GtkWidget *dialog;
+      GtkWindow *window;
+
+      window = gtk_application_get_active_window (GTK_APPLICATION (g_application_get_default ()));
+      dialog = adw_message_dialog_new (window,
+                                       _("Alert"),
+                                       _("Found and automatically deleted tracking IDs from the pasted link."));
+      adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "close", _("Close"));
+
+      adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "close");
+      adw_message_dialog_set_close_response (ADW_MESSAGE_DIALOG (dialog), "close");
+
+      gtk_window_present (GTK_WINDOW (dialog));
+    }
+    gtk_text_buffer_begin_user_action (self->message_buffer);
+    gtk_text_buffer_delete (self->message_buffer, &start, &end);
+    gtk_text_buffer_insert (self->message_buffer, &start, stripped_message, -1);
+    gtk_text_buffer_end_user_action (self->message_buffer);
+  }
+}
+
+static void
+message_bar_text_view_preddit_changed_cb (ChattyMessageBar *self,
+                                          char             *preedit)
+{
+  gboolean has_text, has_files;
+  g_clear_pointer (&self->preedit, g_free);
+  if (preedit && *preedit)
+    self->preedit = g_strdup (preedit);
+
+  has_text = (gtk_text_buffer_get_char_count (self->message_buffer) > 0) || (self->preedit && *self->preedit);
+  has_files = gtk_revealer_get_reveal_child (GTK_REVEALER (self->attachment_revealer));
+  gtk_widget_set_visible (self->send_message_button, has_text || has_files);
+
+  g_clear_handle_id (&self->save_timeout_id, g_source_remove);
+  if (!self->draft_is_loading)
+    self->save_timeout_id = g_timeout_add (SAVE_TIMEOUT,
+                                           chat_page_save_message_to_db,
+                                           self);
+}
+
 static void
 message_bar_activate (GtkWidget  *widget,
                       const char *action_name,
@@ -467,6 +567,8 @@ chatty_message_bar_class_init (ChattyMessageBarClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, message_bar_input_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, message_bar_send_file_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, message_bar_send_message_button_clicked_cb);
+  gtk_widget_class_bind_template_callback (widget_class, message_bar_text_view_preddit_changed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, message_bar_text_buffer_clipboard_paste_done_cb);
 
   gtk_widget_class_install_action (widget_class, "message-bar.activate", NULL, message_bar_activate);
 }
