@@ -14,15 +14,12 @@
 #endif
 
 #include <glib/gi18n.h>
-#include <ctype.h>
 
-#include "gtk3-to-4.h"
 #include "chatty-avatar.h"
 #include "chatty-enums.h"
 #include "chatty-purple.h"
 #include "chatty-file.h"
 #include "chatty-file-item.h"
-#include "chatty-image-item.h"
 #include "chatty-clock.h"
 #include "chatty-message-row.h"
 #include "chatty-settings.h"
@@ -60,162 +57,6 @@ struct _ChattyMessageRow
 
 G_DEFINE_TYPE (ChattyMessageRow, chatty_message_row, GTK_TYPE_LIST_BOX_ROW)
 
-#define URL_WWW "www"
-#define URL_HTTP "http"
-#define URL_HTTPS "https"
-#define URL_FILE "file"
-
-#define get_url_start(_start, _match, _type, _suffix, _out)             \
-  do {                                                                  \
-    char *_uri = _match - strlen (_type);                               \
-    size_t len = strlen (_type _suffix);                                \
-                                                                        \
-    if (*_out)                                                          \
-      break;                                                            \
-                                                                        \
-    if (_match - _start >= strlen (_type) &&                            \
-        g_ascii_strncasecmp (_uri, _type _suffix, len) == 0 &&          \
-        (isalnum (_uri[len]) || _uri[len] == '/'))                      \
-      *_out = match - strlen (_type);                                   \
-  } while (0)
-
-static char *
-find_url (const char  *buffer,
-          char       **end)
-{
-  char *match, *url = NULL;
-  const char *start;
-
-  start = buffer;
-
-  /*
-   * linkify http://,  https://, file://, and www.
-   */
-  while ((match = strpbrk (start, ":."))) {
-    get_url_start (start, match, URL_HTTP, "://", &url);
-    get_url_start (start, match, URL_HTTPS, "://", &url);
-    get_url_start (start, match, URL_FILE, "://", &url);
-    get_url_start (start, match, URL_WWW, ".", &url);
-
-    start = match + 1;
-
-    if (url && url > buffer &&
-        !isspace (url[-1]) && !ispunct (url[-1]))
-      url = NULL;
-
-    if (url)
-      break;
-  }
-
-  if (url) {
-    size_t url_end;
-
-    url_end = strcspn (url, " \n()[],\t\r");
-    if (url_end)
-      *end = url + url_end;
-    else
-      *end = url + strlen (url);
-  }
-
-  return url;
-}
-
-static char **
-load_tracking_ids (void)
-{
-  GBytes *resource_data;
-  char **lines;
-  g_autoptr(GError) error = NULL;
-  resource_data = g_resources_lookup_data ("/sm/puri/Chatty/tracking_ids/tracking_ids.txt",
-                                          G_RESOURCE_LOOKUP_FLAGS_NONE,
-                                          &error);
-  if (error) {
-    g_warning ("Failed to load resource %s", error->message);
-    return NULL;
-  }
-
-  lines = g_strsplit_set (g_bytes_get_data (resource_data, NULL), "\r\n", 0);
-
-  g_bytes_unref(resource_data);
-
-  return lines;
-}
-
-/*
- * https://datatracker.ietf.org/doc/html/rfc1738#section-3.3
- * An URL takes the form: http://<host>:<port>/<path>?<searchpart>
- *
- */
-static char *
-strip_utm (const char *url_to_parse)
-{
-  g_autoptr(GError) error = NULL;
-  GUri *url = NULL;
-  GString *str = NULL;
-  char *stripped_url, *unowned_attr, *unowned_value;
-  GUriParamsIter iter;
-  char **tracking_ids = NULL;
-
-  if (!url_to_parse || !strstr (url_to_parse, "?"))
-    return g_strdup (url_to_parse);
-
-  url = g_uri_parse (url_to_parse, G_URI_FLAGS_NONE, NULL);
-  if (!url)
-    return g_strdup (url_to_parse);
-
-  tracking_ids = load_tracking_ids ();
-  if (!tracking_ids)
-    return g_strdup (url_to_parse);
-
-  str = g_string_new (NULL);
-  g_uri_params_iter_init (&iter, g_uri_get_query (url), -1, "&", G_URI_PARAMS_NONE);
-  while (g_uri_params_iter_next (&iter, &unowned_attr, &unowned_value, &error)) {
-    g_autofree char *attr = g_steal_pointer (&unowned_attr);
-    g_autofree char *value = g_steal_pointer (&unowned_value);
-    gboolean tracking_id = FALSE;
-
-    for (unsigned int j = 0; tracking_ids[j] != NULL; j++) {
-       /* The g_resource has "//" as comments, ignore those */
-       if (g_str_has_prefix (tracking_ids[j], "//"))
-          continue;
-
-       if (g_strcmp0 ((char *) attr, tracking_ids[j]) == 0) {
-         tracking_id = TRUE;
-         break;
-       }
-    }
-
-    if (!tracking_id) {
-      str = g_string_append (str, attr);
-      str = g_string_append (str, "=");
-      str = g_string_append (str, value);
-      str = g_string_append (str, "&");
-    }
-
-  }
-  if (error)
-    str = g_string_append (str, g_uri_get_query (url));
-
-  /* Remove trailing "&" */
-  if (str->len > 1 && str->str[str->len - 1] == '&')
-    g_string_truncate (str, str->len - 1);
-
-  stripped_url = g_uri_join (G_URI_FLAGS_NONE,
-                             g_uri_get_scheme (url),
-                             g_uri_get_userinfo (url),
-                             g_uri_get_host (url),
-                             g_uri_get_port (url),
-                             g_uri_get_path (url),
-                             str->str,
-                             g_uri_get_fragment (url));
-
-  g_uri_unref (url);
-  g_string_free (str, TRUE);
-  g_strfreev (tracking_ids);
-
-  return stripped_url;
-}
-
 static char *
 text_item_linkify (const char *message)
 {
@@ -230,7 +71,7 @@ text_item_linkify (const char *message)
   link_str = g_string_sized_new (256);
   start = end = (char *)message;
 
-  while ((url = find_url (start, &end))) {
+  while ((url = chatty_utils_find_url (start, &end))) {
     ChattySettings *settings;
     g_autofree char *link = NULL;
     g_autofree char *escaped_link = NULL;
@@ -244,7 +85,7 @@ text_item_linkify (const char *message)
 
     link = g_strndup (url, end - url);
     if (chatty_settings_get_strip_url_tracking_ids (settings))
-      utm_stripped_link = strip_utm (link);
+      utm_stripped_link = chatty_utils_strip_utm_from_url (link);
     else
       utm_stripped_link = g_strdup (link);
 
@@ -412,6 +253,44 @@ chatty_message_row_update_footer (ChattyMessageRow *self)
   return G_SOURCE_CONTINUE;
 }
 
+static void
+chatty_message_row_copy_message_text (GtkWidget  *widget,
+                                           const char *action_name,
+                                           GVariant   *param)
+{
+  ChattySettings  *settings;
+  ChattyMessageRow *self = CHATTY_MESSAGE_ROW (widget);
+  GdkClipboard *clipboard;
+  const char *text;
+
+  g_assert (CHATTY_IS_MESSAGE_ROW (self));
+
+  text = chatty_message_get_text (self->message);
+
+  if (!text || !*text)
+    return;
+
+  clipboard = gdk_display_get_clipboard (gdk_display_get_default ());
+
+  settings = chatty_settings_get_default ();
+  if (chatty_settings_get_strip_url_tracking_ids (settings)) {
+    g_autofree char *stripped_message = NULL;
+
+    stripped_message = chatty_utils_strip_utm_from_message (text);
+    gdk_clipboard_set_text (clipboard, stripped_message);
+  } else
+    gdk_clipboard_set_text (clipboard, text);
+}
+
+static void
+long_pressed (GtkGestureLongPress *gesture,
+              gdouble              x,
+              gdouble              y,
+              ChattyMessageRow    *self)
+{
+  gtk_popover_popup (GTK_POPOVER (self->popover));
+}
+
 static void
 message_row_update_message (ChattyMessageRow *self)
 {
@@ -470,14 +349,16 @@ chatty_message_row_class_init (ChattyMessageRowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, ChattyMessageRow, avatar_image);
   gtk_widget_class_bind_template_child (widget_class, ChattyMessageRow, hidden_box);
   gtk_widget_class_bind_template_child (widget_class, ChattyMessageRow, author_label);
+  gtk_widget_class_bind_template_child (widget_class, ChattyMessageRow, popover);
 
   gtk_widget_class_bind_template_child (widget_class, ChattyMessageRow, message_content);
   gtk_widget_class_bind_template_child (widget_class, ChattyMessageRow, files_box);
   gtk_widget_class_bind_template_child (widget_class, ChattyMessageRow, content_separator);
   gtk_widget_class_bind_template_child (widget_class, ChattyMessageRow, message_title);
   gtk_widget_class_bind_template_child (widget_class, ChattyMessageRow, message_body);
-
   gtk_widget_class_bind_template_child (widget_class, ChattyMessageRow, footer_label);
+
+  gtk_widget_class_install_action (widget_class, "win.copy-text", NULL, chatty_message_row_copy_message_text);
 }
 
 static void
@@ -490,11 +371,9 @@ static void
 message_row_add_files (ChattyMessageRow *self)
 {
   GList *files;
-  ChattySettings *settings;
 
   g_assert (CHATTY_IS_MESSAGE_ROW (self));
 
-  settings = chatty_settings_get_default ();
   files = chatty_message_get_files (self->message);
   gtk_widget_set_visible (self->files_box, files && files->data);
 
@@ -505,14 +384,7 @@ message_row_add_files (ChattyMessageRow *self)
     g_assert (CHATTY_IS_FILE (file->data));
     mime_type = chatty_file_get_mime_type (file->data);
 
-    if (!chatty_settings_get_render_attachments (settings)) {
-      child = chatty_file_item_new (self->message, file->data, mime_type);
-    } else if ((mime_type && g_str_has_prefix (mime_type, "image")) ||
-        chatty_message_get_msg_type (self->message) == CHATTY_MESSAGE_IMAGE) {
-      child = chatty_image_item_new (self->message, file->data);
-    } else {
-      child = chatty_file_item_new (self->message, file->data, mime_type);
-    }
+    child = chatty_file_item_new (self->message, file->data, mime_type);
 
     gtk_widget_set_visible (child, TRUE);
     gtk_box_append (GTK_BOX (self->files_box), child);
@@ -525,7 +397,6 @@ chatty_message_row_new (ChattyMessage  *message,
                         gboolean        is_im)
 {
   ChattyMessageRow *self;
-  GtkStyleContext *sc;
   const char *text, *subject;
   ChattyMsgDirection direction;
 
@@ -537,12 +408,23 @@ chatty_message_row_new (ChattyMessage  *message,
   self->message = g_object_ref (message);
   self->is_im = !!is_im;
   direction = chatty_message_get_msg_direction (message);
-  sc = gtk_widget_get_style_context (self->message_content);
 
   message_row_add_files (self);
 
   subject = chatty_message_get_subject (message);
   text = chatty_message_get_text (message);
+
+  if (text && *text) {
+    GtkGesture *gesture;
+    gesture = gtk_gesture_long_press_new ();
+    /*
+     * gtk_widget_add_controller () transfers ownership of the gesture to
+     * ChattyMessageRow so you will not have to worry about freeing it manually
+     */
+    gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (gesture));
+    g_signal_connect (gesture, "pressed", G_CALLBACK (long_pressed), self);
+  }
+
   gtk_widget_set_visible (self->message_title, subject && *subject);
   gtk_widget_set_visible (self->message_body, text && *text);
 
@@ -583,17 +465,17 @@ chatty_message_row_new (ChattyMessage  *message,
   text_item_update_quotes (self);
 
   if (direction == CHATTY_DIRECTION_IN) {
-    gtk_style_context_add_class (sc, "bubble_white");
+    gtk_widget_add_css_class (self->message_content, "bubble_white");
     gtk_widget_set_halign (self->files_box, GTK_ALIGN_START);
     gtk_widget_set_halign (self->content_grid, GTK_ALIGN_START);
     gtk_widget_set_halign (self->message_content, GTK_ALIGN_START);
     gtk_widget_set_halign (self->author_label, GTK_ALIGN_START);
   } else if (direction == CHATTY_DIRECTION_OUT && protocol == CHATTY_PROTOCOL_MMS_SMS) {
-    gtk_style_context_add_class (sc, "bubble_green");
+    gtk_widget_add_css_class (self->message_content, "bubble_green");
   } else if (direction == CHATTY_DIRECTION_OUT) {
-    gtk_style_context_add_class (sc, "bubble_blue");
+    gtk_widget_add_css_class (self->message_content, "bubble_blue");
   } else { /* System message */
-    gtk_style_context_add_class (sc, "bubble_purple");
+    gtk_widget_add_css_class (self->message_content, "bubble_purple");
     gtk_widget_set_hexpand (self->message_content, TRUE);
   }
 
