@@ -37,6 +37,9 @@ gboolean        chatty_settings_get_send_receipts            (ChattySettings *se
 gboolean        chatty_settings_get_message_carbons          (ChattySettings *self);
 gboolean        chatty_settings_get_send_typing              (ChattySettings *self);
 gboolean        chatty_settings_get_strip_url_tracking_ids   (ChattySettings *self);
+gboolean        chatty_settings_get_strip_url_tracking_ids_dialog   (ChattySettings *self);
+void            chatty_settings_set_strip_url_tracking_ids_dialog   (ChattySettings *self,
+                                                                     gboolean shown);
 gboolean        chatty_settings_get_render_attachments       (ChattySettings *self);
 gboolean        chatty_settings_get_blur_idle_buddies        (ChattySettings *self);
 gboolean        chatty_settings_get_indicate_unknown_contacts (ChattySettings *self);
