@@ -20,6 +20,7 @@
 #include "chatty-phone-utils.h"
 #include "chatty-utils.h"
 #include <libebook-contacts/libebook-contacts.h>
+#include <ctype.h>
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-desktop-thumbnail.h>
 
@@ -29,6 +30,222 @@
 #define ASCII_CAPS  "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 #define ASCII_SMALL "abcdefghijklmnopqrstuvwxyz"
 
+#define URL_WWW "www"
+#define URL_HTTP "http"
+#define URL_HTTPS "https"
+#define URL_FILE "file"
+
+#define get_url_start(_start, _match, _type, _suffix, _out)             \
+  do {                                                                  \
+    char *_uri = _match - strlen (_type);                               \
+    size_t len = strlen (_type _suffix);                                \
+                                                                        \
+    if (*_out)                                                          \
+      break;                                                            \
+                                                                        \
+    if (_match - _start >= strlen (_type) &&                            \
+        g_ascii_strncasecmp (_uri, _type _suffix, len) == 0 &&          \
+        (isalnum (_uri[len]) || _uri[len] == '/'))                      \
+      *_out = match - strlen (_type);                                   \
+  } while (0)
+
+char *
+chatty_utils_find_url (const char  *buffer,
+                       char       **end)
+{
+  char *match, *url = NULL;
+  const char *start;
+
+  start = buffer;
+
+  /*
+   * linkify http://,  https://, file://, and www.
+   */
+  while ((match = strpbrk (start, ":."))) {
+    get_url_start (start, match, URL_HTTP, "://", &url);
+    get_url_start (start, match, URL_HTTPS, "://", &url);
+    get_url_start (start, match, URL_FILE, "://", &url);
+    get_url_start (start, match, URL_WWW, ".", &url);
+
+    start = match + 1;
+
+    if (url && url > buffer &&
+        !isspace (url[-1]) && !ispunct (url[-1]))
+      url = NULL;
+
+    if (url)
+      break;
+  }
+
+  if (url) {
+    size_t url_end;
+
+    url_end = strcspn (url, " \n()[],\t\r");
+    if (url_end == strlen (url))
+      *end = url + url_end;
+    else {
+      char *ptr = url + url_end + 1;
+
+      while (ptr[-1] == '(') {
+        size_t count = 0;
+        size_t other_count = 0;
+
+        count = strcspn (ptr, ")");
+        other_count = strcspn (ptr, " \n([],\t\r");
+        if (count < strlen (ptr) && count < other_count)
+          ptr += count + 2;
+        else
+          break;
+      }
+
+      while (!isspace(ptr[-1]) && !isspace(ptr[0]) &&
+             ptr[-1] != '(' && ptr[-1] != ')') {
+        size_t count = 0;
+
+        count = strcspn (ptr, " \n()[],\t\r");
+        ptr += count + 1;
+        if (count == strlen (ptr - count - 1))
+          break;
+      }
+      *end = ptr - 1;
+    }
+  }
+
+  return url;
+}
+
+static char **
+load_tracking_ids (void)
+{
+  GBytes *resource_data;
+  char **lines;
+  g_autoptr(GError) error = NULL;
+  resource_data = g_resources_lookup_data ("/sm/puri/Chatty/tracking_ids/tracking_ids.txt",
+                                          G_RESOURCE_LOOKUP_FLAGS_NONE,
+                                          &error);
+  if (error) {
+    g_warning ("Failed to load resource %s", error->message);
+    return NULL;
+  }
+
+  lines = g_strsplit_set (g_bytes_get_data (resource_data, NULL), "\r\n", 0);
+
+  g_bytes_unref(resource_data);
+
+  return lines;
+}
+
+/*
+ * https://datatracker.ietf.org/doc/html/rfc1738#section-3.3
+ * An URL takes the form: http://<host>:<port>/<path>?<searchpart>
+ *
+ */
+char *
+chatty_utils_strip_utm_from_url (const char *url_to_parse)
+{
+  g_autoptr(GError) error = NULL;
+  g_autoptr(GUri) url = NULL;
+  GString *str = NULL;
+  char *stripped_url, *unowned_attr, *unowned_value;
+  GUriParamsIter iter;
+  char **tracking_ids = NULL;
+
+  if (!url_to_parse || !strstr (url_to_parse, "?"))
+    return g_strdup (url_to_parse);
+
+  url = g_uri_parse (url_to_parse, G_URI_FLAGS_NONE, NULL);
+  if (!url)
+    return g_strdup (url_to_parse);
+
+  tracking_ids = load_tracking_ids ();
+  if (!tracking_ids)
+    return g_strdup (url_to_parse);
+
+  str = g_string_new (NULL);
+  g_uri_params_iter_init (&iter, g_uri_get_query (url), -1, "&", G_URI_PARAMS_NONE);
+  while (g_uri_params_iter_next (&iter, &unowned_attr, &unowned_value, &error)) {
+    g_autofree char *attr = g_steal_pointer (&unowned_attr);
+    g_autofree char *value = g_steal_pointer (&unowned_value);
+    gboolean tracking_id = FALSE;
+
+    for (unsigned int j = 0; tracking_ids[j] != NULL; j++) {
+       /* The g_resource has "//" as comments, ignore those */
+       if (g_str_has_prefix (tracking_ids[j], "//"))
+          continue;
+
+       if (g_strcmp0 ((char *) attr, tracking_ids[j]) == 0) {
+         tracking_id = TRUE;
+         break;
+       }
+    }
+
+    if (!tracking_id) {
+      str = g_string_append (str, attr);
+      str = g_string_append (str, "=");
+      str = g_string_append (str, value);
+      str = g_string_append (str, "&");
+    }
+
+  }
+  if (error)
+    str = g_string_append (str, g_uri_get_query (url));
+
+  /* Remove trailing "&" */
+  if (str->len > 1 && str->str[str->len - 1] == '&')
+    g_string_truncate (str, str->len - 1);
+
+  stripped_url = g_uri_join (G_URI_FLAGS_NONE,
+                             g_uri_get_scheme (url),
+                             g_uri_get_userinfo (url),
+                             g_uri_get_host (url),
+                             g_uri_get_port (url),
+                             g_uri_get_path (url),
+                             (str->len ? str->str : NULL),
+                             g_uri_get_fragment (url));
+
+  g_string_free (str, TRUE);
+  g_strfreev (tracking_ids);
+
+  return stripped_url;
+}
+
+char *
+chatty_utils_strip_utm_from_message (const char *message)
+{
+  GString *str = NULL;
+  char *start, *end, *url;
+
+  if (!message)
+    return NULL;
+
+  if (!*message)
+    return g_strdup ("");
+
+  str = g_string_sized_new (256);
+  start = end = (char *)message;
+
+  while ((url = chatty_utils_find_url (start, &end))) {
+    g_autofree char *link = NULL;
+    g_autofree char *escaped_link = NULL;
+    g_autofree char *utm_stripped_link = NULL;
+
+    link = g_strndup (start, url - start);
+    g_string_append (str, link);
+    g_free (link);
+
+    link = g_strndup (url, end - url);
+    utm_stripped_link = chatty_utils_strip_utm_from_url (link);
+    g_string_append (str, utm_stripped_link);
+    start = end;
+  }
+
+  if (start && *start) {
+    g_string_append (str, start);
+  }
+
+  return g_string_free (str, FALSE);
+}
+
 /*
  * matrix_id_is_valid:
  * @name: A string
@@ -533,4 +750,3 @@ chatty_utils_create_thumbnail_finish (GAsyncResult  *result,
 
   return g_task_propagate_boolean (G_TASK (result), error);
 }
-
