@@ -42,7 +42,7 @@ struct _ChattyWindow
 
   GtkWidget *chat_list;
 
-  GtkWidget *content_box;
+  GtkWidget *split_view;
   GtkWidget *side_bar;
   GtkWidget *main_view;
 
@@ -102,9 +102,8 @@ window_set_item (ChattyWindow *self,
                                                           G_CALLBACK (window_chat_changed_cb),
                                                           self, G_CONNECT_SWAPPED);
   }
-
   if (!chat)
-    adw_leaflet_set_visible_child (ADW_LEAFLET (self->content_box), self->side_bar);
+    adw_navigation_split_view_set_show_content (ADW_NAVIGATION_SPLIT_VIEW (self->split_view), FALSE);
 
   chatty_window_set_item (self, CHATTY_ITEM (chat));
 }
@@ -164,10 +163,12 @@ window_chat_list_selection_changed (ChattyWindow   *self,
   g_return_if_fail (CHATTY_IS_CHAT (chat));
 
   if (chat == (gpointer)chatty_main_view_get_item (CHATTY_MAIN_VIEW (self->main_view))) {
-    adw_leaflet_set_visible_child (ADW_LEAFLET (self->content_box), self->main_view);
+    adw_navigation_split_view_set_show_content (ADW_NAVIGATION_SPLIT_VIEW (self->split_view), TRUE);
 
-    if (chatty_window_get_active_chat (self))
+    if (chatty_window_get_active_chat (self)) {
       chatty_chat_set_unread_count (chat, 0);
+      chatty_chat_withdraw_notification (chat);
+    }
 
     return;
   }
@@ -180,23 +181,32 @@ window_chat_list_selection_changed (ChattyWindow   *self,
     chatty_window_open_chat (self, chat);
 }
 
+/*
+ * adw_navigation_split_view_get_collapsed () doesn't update fast enough for the
+ * signal. Thus, I just seperate the signals and do logic accordingly
+ */
 static void
 notify_fold_cb (ChattyWindow *self)
 {
-  gboolean folded;
-
   if (!gtk_widget_get_mapped (GTK_WIDGET (self)))
     return;
 
-  folded = adw_leaflet_get_folded (ADW_LEAFLET (self->content_box));
-  chatty_chat_list_set_selection_mode (CHATTY_CHAT_LIST (self->chat_list), !folded);
+  chatty_chat_list_set_selection_mode (CHATTY_CHAT_LIST (self->chat_list), FALSE);
+  chatty_side_bar_set_show_end_title_buttons (CHATTY_SIDE_BAR (self->side_bar), TRUE);
+}
 
-  if (!folded) {
-    ChattyItem *item;
+static void
+notify_unfold_cb (ChattyWindow *self)
+{
+  ChattyItem *item = NULL;
+  if (!gtk_widget_get_mapped (GTK_WIDGET (self)))
+    return;
 
-    item = chatty_main_view_get_item (CHATTY_MAIN_VIEW (self->main_view));
-    chatty_chat_list_select_item (CHATTY_CHAT_LIST (self->chat_list), item);
-  }
+  chatty_chat_list_set_selection_mode (CHATTY_CHAT_LIST (self->chat_list), TRUE);
+
+  item = chatty_main_view_get_item (CHATTY_MAIN_VIEW (self->main_view));
+  chatty_chat_list_select_item (CHATTY_CHAT_LIST (self->chat_list), item);
+  chatty_side_bar_set_show_end_title_buttons (CHATTY_SIDE_BAR (self->side_bar), FALSE);
 }
 
 static void
@@ -314,16 +324,16 @@ chatty_window_unarchive_chat (GtkWidget  *widget,
 }
 
 static void
-window_block_chat_response_cb (ChattyWindow *self,
-                               int           response_id,
-                               GtkDialog    *dialog)
+window_block_chat_response_cb (AdwMessageDialog *dialog,
+                               const char       *response,
+                               gpointer          user_data)
+
 {
+  ChattyWindow *self = CHATTY_WINDOW (user_data);
+  g_autoptr(GError) error = NULL;
 
   g_assert (CHATTY_IS_WINDOW (self));
-
-  gtk_window_destroy (GTK_WINDOW (dialog));
-
-  if (response_id == GTK_RESPONSE_OK) {
+  if (g_strcmp0 (response, "block") == 0) {
     ChattyItem *item;
 
     item = chatty_main_view_get_item (CHATTY_MAIN_VIEW (self->main_view));
@@ -337,8 +347,9 @@ chatty_window_block_chat (GtkWidget  *widget,
                           GVariant   *param)
 {
   ChattyWindow *self = CHATTY_WINDOW (widget);
-  GtkWidget *message;
   ChattyItem *item;
+  GtkWindow *window;
+  GtkWidget *dialog;
 
   g_assert (CHATTY_IS_WINDOW (self));
 
@@ -346,17 +357,24 @@ chatty_window_block_chat (GtkWidget  *widget,
   g_return_if_fail (CHATTY_IS_MM_CHAT (item));
   g_return_if_fail (chatty_chat_is_im (CHATTY_CHAT (item)));
 
-  message = gtk_message_dialog_new (GTK_WINDOW (self),
-                                    GTK_DIALOG_MODAL | GTK_DIALOG_USE_HEADER_BAR,
-                                    GTK_MESSAGE_INFO,
-                                    GTK_BUTTONS_OK_CANCEL,
-                                    _("You shall no longer be notified for new messages, continue?"));
+  window = gtk_application_get_active_window (GTK_APPLICATION (g_application_get_default ()));
+  dialog = adw_message_dialog_new (window,
+                                   _("Alert"),
+                                   _("You shall no longer be notified for new messages, continue?"));
 
-  g_signal_connect_object (message, "response",
-                           G_CALLBACK (window_block_chat_response_cb),
-                           self, G_CONNECT_SWAPPED);
+  adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
+                                    "cancel",  _("_Cancel"),
+                                    "block", _("Block"),
+                                    NULL);
 
-  gtk_window_present (GTK_WINDOW (message));
+  adw_message_dialog_set_response_appearance (ADW_MESSAGE_DIALOG (dialog), "block", ADW_RESPONSE_DESTRUCTIVE);
+
+  adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "cancel");
+  adw_message_dialog_set_close_response (ADW_MESSAGE_DIALOG (dialog), "cancel");
+
+  g_signal_connect (dialog, "response", G_CALLBACK (window_block_chat_response_cb), self);
+
+  gtk_window_present (GTK_WINDOW (dialog));
 }
 
 static void
@@ -378,14 +396,16 @@ chatty_window_unblock_chat (GtkWidget  *widget,
 }
 
 static void
-window_delete_chat_response_cb (ChattyWindow *self,
-                                int           response_id,
-                                GtkDialog    *dialog)
+window_delete_chat_response_cb (AdwMessageDialog *dialog,
+                                const char       *response,
+                                gpointer          user_data)
 {
-  g_assert (CHATTY_IS_WINDOW (self));
-  g_assert (GTK_IS_DIALOG (dialog));
+  ChattyWindow *self = CHATTY_WINDOW (user_data);
+  g_autoptr(GError) error = NULL;
 
-  if (response_id == GTK_RESPONSE_OK) {
+  g_assert (CHATTY_IS_WINDOW (self));
+
+  if (g_strcmp0 (response, "delete") == 0) {
     ChattyChat *chat;
 
     chat = (ChattyChat *)chatty_main_view_get_item (CHATTY_MAIN_VIEW (self->main_view));
@@ -405,11 +425,9 @@ window_delete_chat_response_cb (ChattyWindow *self,
 
     window_set_item (self, NULL);
 
-    if (!adw_leaflet_get_folded (ADW_LEAFLET (self->content_box)))
+    if (!adw_navigation_split_view_get_collapsed (ADW_NAVIGATION_SPLIT_VIEW (self->split_view)))
       chatty_chat_list_select_first (CHATTY_CHAT_LIST (self->chat_list));
   }
-
-  gtk_window_destroy (GTK_WINDOW (dialog));
 }
 
 static void
@@ -418,11 +436,11 @@ chatty_window_delete_chat (GtkWidget  *widget,
                            GVariant   *param)
 {
   ChattyWindow *self = CHATTY_WINDOW (widget);
-  g_autofree char *text = NULL;
-  GtkWidget *dialog;
   ChattyChat *chat;
   const char *name;
-  const char *sub_text;
+  GtkWindow *window;
+  GtkWidget *dialog;
+  g_autofree char *secondary_text = NULL;
 
   g_assert (CHATTY_IS_WINDOW (self));
 
@@ -431,32 +449,28 @@ chatty_window_delete_chat (GtkWidget  *widget,
 
   name = chatty_item_get_name (CHATTY_ITEM (chat));
 
+  window = gtk_application_get_active_window (GTK_APPLICATION (g_application_get_default ()));
+  dialog = adw_message_dialog_new (window, NULL, NULL);
+
   if (chatty_chat_is_im (chat)) {
-    text = g_strdup_printf (_("Delete chat with “%s”"), name);
-    sub_text = _("This deletes the conversation history");
+    adw_message_dialog_format_heading (ADW_MESSAGE_DIALOG (dialog), _("Delete chat with “%s”"), name);
+    adw_message_dialog_format_body (ADW_MESSAGE_DIALOG (dialog), _("This deletes the conversation history"));
   } else {
-    text = g_strdup_printf (_("Disconnect group chat “%s”"), name);
-    sub_text = _("This removes chat from chats list");
+    adw_message_dialog_format_heading (ADW_MESSAGE_DIALOG (dialog), _("Disconnect group chat “%s”"), name);
+    adw_message_dialog_format_body (ADW_MESSAGE_DIALOG (dialog), _("This removes chat from chats list"));
   }
 
-  dialog = gtk_message_dialog_new (GTK_WINDOW (self),
-                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_QUESTION,
-                                   GTK_BUTTONS_NONE,
-                                   "%s", text);
+  adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
+                                    "cancel",  _("_Cancel"),
+                                    "delete", _("Delete"),
+                                    NULL);
 
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                          _("Cancel"), GTK_RESPONSE_CANCEL,
-                          _("Delete"), GTK_RESPONSE_OK,
-                          NULL);
+  adw_message_dialog_set_response_appearance (ADW_MESSAGE_DIALOG (dialog), "delete", ADW_RESPONSE_DESTRUCTIVE);
 
-  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                            "%s", sub_text);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
+  adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "cancel");
+  adw_message_dialog_set_close_response (ADW_MESSAGE_DIALOG (dialog), "cancel");
 
-  g_signal_connect_object (dialog, "response",
-                           G_CALLBACK (window_delete_chat_response_cb),
-                           self, G_CONNECT_SWAPPED);
+  g_signal_connect (dialog, "response", G_CALLBACK (window_delete_chat_response_cb), self);
 
   gtk_window_present (GTK_WINDOW (dialog));
 }
@@ -483,7 +497,7 @@ chatty_window_leave_chat (GtkWidget  *widget,
 
   window_set_item (self, NULL);
 
-  if (!adw_leaflet_get_folded (ADW_LEAFLET (self->content_box)))
+  if (!adw_navigation_split_view_get_collapsed (ADW_NAVIGATION_SPLIT_VIEW (self->split_view)))
     chatty_chat_list_select_first (CHATTY_CHAT_LIST (self->chat_list));
 }
 
@@ -539,6 +553,13 @@ chatty_window_show_settings (GtkWidget  *widget,
   gtk_window_present (GTK_WINDOW (self->settings_dialog));
 }
 
+static void
+window_chat_list_hiding (ChattyWindow *self)
+{
+  if (!chatty_chat_list_is_archived (CHATTY_CHAT_LIST (self->chat_list)))
+    window_set_item (self, NULL);
+}
+
 static void
 chatty_window_go_back (GtkWidget  *widget,
                        const char *action_name,
@@ -548,8 +569,7 @@ chatty_window_go_back (GtkWidget  *widget,
 
   g_assert (CHATTY_IS_WINDOW (self));
 
-  if (!chatty_chat_list_is_archived (CHATTY_CHAT_LIST (self->chat_list)))
-    window_set_item (self, NULL);
+  window_chat_list_hiding (self);
 }
 
 static void
@@ -597,6 +617,8 @@ chatty_window_call_user (GtkWidget  *widget,
 {
   ChattyWindow *self = CHATTY_WINDOW (widget);
   g_autofree char *uri = NULL;
+  g_autoptr(GtkUriLauncher) uri_launcher = NULL;
+  GtkWindow *window;
   ChattyChat *chat;
 
   g_assert (CHATTY_IS_WINDOW (self));
@@ -607,7 +629,10 @@ chatty_window_call_user (GtkWidget  *widget,
   uri = g_strconcat ("tel://", chatty_chat_get_chat_name (chat), NULL);
 
   CHATTY_INFO (uri, "Calling uri:");
-  gtk_show_uri (NULL, uri, GDK_CURRENT_TIME);
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (g_application_get_default ()));
+  uri_launcher = gtk_uri_launcher_new (uri);
+  gtk_uri_launcher_launch (uri_launcher, window, NULL, NULL, NULL);
 }
 
 static void
@@ -707,12 +732,14 @@ chatty_window_class_init (ChattyWindowClass *klass)
                                                "/sm/puri/Chatty/"
                                                "ui/chatty-window.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, content_box);
+  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, split_view);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, side_bar);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, main_view);
 
   gtk_widget_class_bind_template_callback (widget_class, notify_fold_cb);
+  gtk_widget_class_bind_template_callback (widget_class, notify_unfold_cb);
   gtk_widget_class_bind_template_callback (widget_class, window_chat_list_selection_changed);
+  gtk_widget_class_bind_template_callback (widget_class, window_chat_list_hiding);
 
   gtk_widget_class_install_action (widget_class, "win.new-chat", NULL, chatty_window_start_new_chat);
   gtk_widget_class_install_action (widget_class, "win.new-sms-mms", NULL, chatty_window_start_sms_mms_chat);
@@ -735,6 +762,7 @@ chatty_window_class_init (ChattyWindowClass *klass)
 static void
 chatty_window_init (ChattyWindow *self)
 {
+  gboolean folded;
   gtk_widget_init_template (GTK_WIDGET (self));
 
   self->chat_list = chatty_side_bar_get_chat_list (CHATTY_SIDE_BAR (self->side_bar));
@@ -744,6 +772,10 @@ chatty_window_init (ChattyWindow *self)
   g_signal_connect_object (self->manager, "chat-deleted",
                            G_CALLBACK (window_chat_deleted_cb), self,
                            G_CONNECT_SWAPPED);
+
+  folded = adw_navigation_split_view_get_collapsed (ADW_NAVIGATION_SPLIT_VIEW (self->split_view));
+  chatty_side_bar_set_show_end_title_buttons (CHATTY_SIDE_BAR (self->side_bar), folded);
+  chatty_main_view_set_item (CHATTY_MAIN_VIEW (self->main_view), NULL);
 }
 
 GtkWidget *
@@ -793,10 +825,12 @@ chatty_window_open_chat (ChattyWindow *self,
   CHATTY_INFO (chatty_chat_get_chat_name (chat),
                "opening chat (%p), type: %s, chat-name:", chat, G_OBJECT_TYPE_NAME (chat));
 
+  adw_navigation_split_view_set_show_content (ADW_NAVIGATION_SPLIT_VIEW (self->split_view), TRUE);
+
   window_set_item (self, chat);
 
-  adw_leaflet_set_visible_child (ADW_LEAFLET (self->content_box), self->main_view);
-
-  if (chatty_window_get_active_chat (self))
+  if (chatty_window_get_active_chat (self)) {
     chatty_chat_set_unread_count (chat, 0);
+    chatty_chat_withdraw_notification (chat);
+  }
 }
