@@ -16,7 +16,6 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 
-#include "gtk3-to-4.h"
 #include "chatty-manager.h"
 #include "chatty-chat.h"
 #include "chatty-purple.h"
@@ -204,8 +203,9 @@ open_contacts_finish_cb (GObject      *object,
 {
   ChattyNewChatDialog *self = user_data;
   ChattyEds *chatty_eds = (ChattyEds *)object;
-  GtkWidget *dialog;
+  GtkWindow *window;
   g_autoptr(GError) error = NULL;
+  GtkWidget *dialog;
 
   g_assert (CHATTY_IS_NEW_CHAT_DIALOG (self));
   g_assert (CHATTY_IS_EDS (chatty_eds));
@@ -217,14 +217,17 @@ open_contacts_finish_cb (GObject      *object,
   if (!error)
     return;
 
-  dialog = gtk_message_dialog_new (GTK_WINDOW (self),
-                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_WARNING,
-                                   GTK_BUTTONS_CLOSE,
-                                   _("Error opening GNOME Contacts: %s"),
-                                   error->message);
-  g_signal_connect_swapped (dialog, "response",
-                            G_CALLBACK (gtk_window_destroy), dialog);
+  window = gtk_application_get_active_window (GTK_APPLICATION (g_application_get_default ()));
+
+  dialog = adw_message_dialog_new (window, _("Error"), NULL);
+  adw_message_dialog_format_body (ADW_MESSAGE_DIALOG (dialog),
+                                  _("Error opening GNOME Contacts: %s"),
+                                  error->message);
+  adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "close", _("Close"));
+
+  adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "close");
+  adw_message_dialog_set_close_response (ADW_MESSAGE_DIALOG (dialog), "close");
+
   gtk_window_present (GTK_WINDOW (dialog));
 }
 
@@ -628,7 +631,7 @@ chatty_new_chat_dialog_init (ChattyNewChatDialog *self)
   gtk_widget_init_template (GTK_WIDGET (self));
   self->cancellable = g_cancellable_new ();
 
-  self->self_contact = CHATTY_ITEM (chatty_contact_dummy_new (_("You"), "MMS"));
+  self->self_contact = CHATTY_ITEM (chatty_contact_dummy_new (_("You"), _("SMS/MMS")));
   chatty_list_row_set_item (CHATTY_LIST_ROW (self->self_contact_row), self->self_contact);
 
   self->selection_list = g_list_store_new (CHATTY_TYPE_ITEM);
