@@ -23,12 +23,12 @@
 
 #pragma once
 
-#include <handy.h>
+#include <adwaita.h>
 
 G_BEGIN_DECLS
 
 #define CHATTY_TYPE_SETTINGS_DIALOG (chatty_settings_dialog_get_type())
-G_DECLARE_FINAL_TYPE (ChattySettingsDialog, chatty_settings_dialog, CHATTY, SETTINGS_DIALOG, HdyWindow)
+G_DECLARE_FINAL_TYPE (ChattySettingsDialog, chatty_settings_dialog, CHATTY, SETTINGS_DIALOG, AdwWindow)
 
 GtkWidget *chatty_settings_dialog_new (GtkWindow *parent_window);
 
