@@ -8,8 +8,7 @@
  * SPDX-License-Identifier: GPL-2-or-later OR CC0-1.0
  */
 
-#include <handy.h>
-#include <libgd/gd.h>
+#include <adwaita.h>
 
 #include "dialogs/chatty-chat-info.h"
 
@@ -18,9 +17,8 @@ void __attribute__((constructor)) initialize_libraries (void);
 void
 __attribute__((constructor)) initialize_libraries (void)
 {
-  gtk_init (NULL, NULL);
-  hdy_init ();
-  gd_ensure_types ();
+  gtk_init ();
+  adw_init ();
 
   g_type_ensure (CHATTY_TYPE_CHAT_INFO);
 }
