@@ -14,7 +14,6 @@
 # include "config.h"
 #endif
 
-#include "contrib/gtk.h"
 #define CMATRIX_USE_EXPERIMENTAL_API
 #include "cmatrix.h"
 
@@ -132,7 +131,7 @@ chatty_matrix_init (ChattyMatrix *self)
   self->list_of_chat_list = g_list_store_new (G_TYPE_LIST_MODEL);
 
   model = G_LIST_MODEL (self->list_of_chat_list);
-  self->chat_list = gtk_flatten_list_model_new (CHATTY_TYPE_CHAT, model);
+  self->chat_list = gtk_flatten_list_model_new (g_object_ref (model));
 }
 
 ChattyMatrix *
@@ -221,7 +220,7 @@ chatty_matrix_load (ChattyMatrix *self)
 
   data_path = g_build_filename (g_get_user_data_dir (), "chatty", NULL);
   cache_path = g_build_filename (g_get_user_cache_dir (), "chatty", NULL);
-  self->cm_matrix = cm_matrix_new (data_path, cache_path, "sm.puri.Chatty",
+  self->cm_matrix = cm_matrix_new (data_path, cache_path, CHATTY_APP_ID,
                                    self->disable_auto_login);
   client_list = cm_matrix_get_clients_list (self->cm_matrix);
   g_signal_connect_object (client_list, "items-changed",
