@@ -24,6 +24,8 @@ test_message_text_markup (void)
     char *text;
     char *markup;
   } data;
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wbidi-chars"
   data array[] = {
     {"",""},
     {"abc","abc"},
@@ -67,8 +69,35 @@ test_message_text_markup (void)
     {
      "http://www.example.com/user's-image.png?blah",
      "<a href=\"http://www.example.com/user&apos;s-image.png?blah\">http://www.example.com/user&apos;s-image.png?blah</a>"},
+    {
+     "https://en.wikipedia.org/wiki/Sphere_(venue)",
+     "<a href=\"https://en.wikipedia.org/wiki/Sphere_(venue)\">https://en.wikipedia.org/wiki/Sphere_(venue)</a>"
+    },
+    {
+     "https://example.com(venue hi hi hi)",
+     "<a href=\"https://example.com\">https://example.com</a>(venue hi hi hi)"
+    },
+    {
+     "Open some example website (eg: https://www.example.com)",
+     "Open some example website (eg: <a href=\"https://www.example.com\">https://www.example.com</a>)"
+    },
+    {
+     "https://en.wikipedia.org/wiki/Sphere_(venue) You should check this out",
+     "<a href=\"https://en.wikipedia.org/wiki/Sphere_(venue)\">https://en.wikipedia.org/wiki/Sphere_(venue)</a> You should check this out"
+    },
+    {
+     "https://www.google.com/maps/place/Carnegie+Mellon+University/@40.4432027,-79.9454248,17z/data=!3m1!4b1!4m6!3m5!1s0x8834f21f58679a9f:0x88716b461fc4daf4!8m2!3d40.4432027!4d-79.9428499!16zL20vMGN3eF8?entry=ttu",
+     "<a href=\"https://www.google.com/maps/place/Carnegie+Mellon+University/@40.4432027,-79.9454248,17z/data=!3m1!4b1!4m6!3m5!1s0x8834f21f58679a9f:0x88716b461fc4daf4!8m2!3d40.4432027!4d-79.9428499!16zL20vMGN3eF8?entry=ttu\">https://www.google.com/maps/place/Carnegie+Mellon+University/@40.4432027,-79.9454248,17z/data=!3m1!4b1!4m6!3m5!1s0x8834f21f58679a9f:0x88716b461fc4daf4!8m2!3d40.4432027!4d-79.9428499!16zL20vMGN3eF8?entry=ttu</a>"
+    },
+    {
+     "https://www.google.com/maps/place/Carnegie+Mellon+University/@40.4432027,-79.9454248,17z/data=!3m1!4b1!4m6!3m5!1s0x8834f21f58679a9f:0x88716b461fc4daf4!8m2!3d40.4432027!4d-79.9428499!16zL20vMGN3eF8?entry=ttu This is the spot",
+     "<a href=\"https://www.google.com/maps/place/Carnegie+Mellon+University/@40.4432027,-79.9454248,17z/data=!3m1!4b1!4m6!3m5!1s0x8834f21f58679a9f:0x88716b461fc4daf4!8m2!3d40.4432027!4d-79.9428499!16zL20vMGN3eF8?entry=ttu\">https://www.google.com/maps/place/Carnegie+Mellon+University/@40.4432027,-79.9454248,17z/data=!3m1!4b1!4m6!3m5!1s0x8834f21f58679a9f:0x88716b461fc4daf4!8m2!3d40.4432027!4d-79.9428499!16zL20vMGN3eF8?entry=ttu</a> This is the spot"
+    },
+    {//https://00xbyte.github.io/posts/Don%27t-Believe-Your-Eyes-A-WhatsApp-Clickjacking-Vulnerability/ Make sure that Chatty isn't vulnerable to this
+     "‮https://moc.margatsni.nl//:sptth",
+     "‮https://moc.margatsni.nl//:sptth"},
   };
-
+#pragma GCC diagnostic pop
   for (guint i = 0; i < G_N_ELEMENTS (array); i++) {
     g_autofree char *content = NULL;
     GtkLabel *label;
@@ -90,45 +119,6 @@ test_message_text_markup (void)
   }
 }
 
-static void
-test_message_strip_utm (void)
-{
-  typedef struct data {
-    char *text;
-    char *check;
-  } data;
-  data array[] = {
-    {
-     "http://www.example.com/user's-image.png?blah=1234",
-     "http://www.example.com/user's-image.png?blah=1234"},
-    {
-     "http://www.example.com/user's-image.png?blah",
-     "http://www.example.com/user's-image.png?blah"},
-    {
-     "http://www.example.com/user's-image.png?utm_source=1234qwer&fbclid=1234564&",
-     "http://www.example.com/user's-image.png?"},
-    {
-     "https://www.example.com/?t=ftsa&q=hello&ia=definition",
-     "https://www.example.com/?t=ftsa&q=hello&ia=definition"},
-    {
-     "http://example.com/utm_source/something?v=_utm_source",
-     "http://example.com/utm_source/something?v=_utm_source"},
-    {
-     "http://www.example.com/user's-image.png?_utm_sourcecode=1234qwer&fbclid=1234564&",
-     "http://www.example.com/user's-image.png?_utm_sourcecode=1234qwer"},
-    {
-     "http://utm_source.example.com/something?wowbraid=123",
-     "http://utm_source.example.com/something?wowbraid=123"},
-  };
-
-  for (guint i = 0; i < G_N_ELEMENTS (array); i++) {
-    g_autofree char *content = NULL;
-
-    content = strip_utm (array[i].text);
-    g_assert_cmpstr (content, ==, array[i].check);
-  }
-}
-
 int
 main (int   argc,
       char *argv[])
@@ -136,7 +126,6 @@ main (int   argc,
   g_test_init (&argc, &argv, NULL);
 
   g_test_add_func ("/message-text/markup", test_message_text_markup);
-  g_test_add_func ("/message-text/strip_utm", test_message_strip_utm);
 
   return g_test_run ();
 }
